unit FrmMidDlg;

{
  BLUE HELL software.

  Copyright (C) 1999 .. 2012, Jan Punter.

  Notes :

    midi componenets needed : http://sourceforge.net/projects/dmidi/
    hires timer needed      : source code included ... I may have stolen that somewhere without
                              acknowleging the source .. somewhere in 1999 or so ...
                              It sure looks a lot like : http://www.programmersheaven.com/download/2818/1/ZipView.aspx
                              but details as to the origin seem absent there too.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  mmSystem, StdCtrls, Buttons, ExtCtrls, Spin;

type

  TUnitsType = (
    utMs,
    utBps,   // Hz ...
    utBpm
  );

  TFormMidiDialog = class(TForm)
    Panel1: TPanel;
    SpeedButtonOk: TSpeedButton;
    SpeedButtonCancel: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    ComboBoxInputs: TComboBox;
    ComboBoxOutputs: TComboBox;
    procedure FormShow(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SpeedButtonOkClick(Sender: TObject);
    procedure SpeedButtonCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
      FInitialized  : Boolean;
      FInSelection  : String;
      FOutSelection : String;
    Procedure   Initialize;
    Procedure   ConfirmDialog;
    Procedure   CancelDialog;
    Procedure   SetInSelection ( Const aValue: String);
    Procedure   SetOutSelection( Const aValue: String);
  Public
    Property    InSelection  : String Read FInSelection  Write SetInSelection;
    Property    OutSelection : String Read FOutSelection Write SetOutSelection;
  end;

var
  FormMidiDialog: TFormMidiDialog;

implementation

{$R *.DFM}

{$WARN UNSAFE_CODE OFF}

// User area

    Procedure   TFormMidiDialog.Initialize;
    Var
      i       : Integer;
      aInCaps : TMidiInCaps;
      aOutCaps: TMidiOutCaps;
    Begin
      If Not FInitialized
      Then Begin
        // Inputs
        With ComboBoxInputs Do
        Begin
          Clear;
          FInSelection := '';
          For i := 0 To Integer( MidiInGetNumDevs) - 1 Do
          Begin
            MidiInGetDevCaps( i, @ aInCaps, SizeOf( aInCaps));
            Items.Add( StrPas( aInCaps.szPName));
          End;
          If Items.Count > 0
          Then InSelection := Items[ 0];
        End;
        // Outputs
        With ComboBoxOutputs Do
        Begin
          Clear;
          FOutSelection := '';
          MidiOutGetDevCaps( MIDI_MAPPER, @ aOutCaps, SizeOf( aOutCaps));
          Items.Add( StrPas( aOutCaps.szPName));
          For i := 0 To MidiOutGetNumDevs - 1 Do
          Begin
            MidiOutGetDevCaps( i, @ aOutCaps, SizeOf( aOutCaps));
            Items.Add( StrPas( aOutCaps.szPName));
          End;
          If Items.Count > 0
          Then OutSelection := Items[ 0];
        End;
        // Flag initialization
        FInitialized := True;
      End;
    End;

    Procedure   TFormMidiDialog.ConfirmDialog;
    Begin
      With ComboBoxInputs Do
        If ItemIndex >= 0
        Then InSelection := Items[ ItemIndex]
        Else InSelection := '';
      With ComboBoxOutputs Do
        If ItemIndex >= 0
        Then OutSelection := Items[ ItemIndex]
        Else OutSelection := '';
      ModalResult  := mrOk;
    End;

    Procedure   TFormMidiDialog.CancelDialog;
    Begin
      With ComboBoxInputs Do
        If ItemIndex >= 0
        Then InSelection := Items[ ItemIndex]
        Else InSelection := '';
      With ComboBoxOutputs Do
        If ItemIndex >= 0
        Then OutSelection := Items[ ItemIndex]
        Else OutSelection := '';
      ModalResult := mrCancel;
    End;

    Procedure   TFormMidiDialog.SetInSelection( Const aValue: String);
    Var
      i : Integer;
    Begin
      With ComboBoxInputs Do
      Begin
        ItemIndex := -1;
        For i := 0 To Items.Count - 1 Do
          If UpperCase( aValue) = UpperCase( Items[ i])
          Then Begin
            ItemIndex := i;
            FInSelection := Items[ i];
            Break;
          End;
      End;
    End;

    Procedure   TFormMidiDialog.SetOutSelection( Const aValue: String);
    Var
      i : Integer;
    Begin
      With ComboBoxOutputs Do
      Begin
        ItemIndex := -1;
        For i := 0 To Items.Count - 1 Do
          If UpperCase( aValue) = UpperCase( Items[ i])
          Then Begin
            ItemIndex := i;
            FOutSelection := Items[ i];
            Break;
          End;
      End;
    End;

// Delphi area


procedure TFormMidiDialog.FormShow(Sender: TObject);
begin
  Initialize;
end;

procedure TFormMidiDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 If Shift = []
 Then
   Case Key Of
     VK_RETURN : ConfirmDialog;
     VK_ESCAPE : CancelDialog;
   End;
end;

procedure TFormMidiDialog.SpeedButtonOkClick(Sender: TObject);
begin
  ConfirmDialog;
end;

procedure TFormMidiDialog.SpeedButtonCancelClick(Sender: TObject);
begin
  CancelDialog;
end;


procedure TFormMidiDialog.FormCreate(Sender: TObject);
begin
  Initialize;
end;

end.
